@@ -6,19 +6,19 @@ use strict;
 use Carp qw( croak );
 
 use vars qw($VERSION);
-$VERSION = '1.269_002'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.294_272'; # NOTE - Should be #.### (three decimal places)
 
 sub import {
   my $self = shift;
 
-  my @loops    = grep(/^Loop\:\:/, @_);
+  my @loops    = grep(/^(?:XS::)?Loop::/, @_);
   my @sessions = grep(/^(Session|NFA)$/, @_);
-  my @modules  = grep(!/^(Kernel|Session|NFA|Loop)$/, @_);
+  my @modules  = grep(!/^(Kernel|Session|NFA|(?:XS::)?Loop::[\w:]+)$/, @_);
 
   croak "can't use multiple event loops at once"
     if (@loops > 1);
   croak "POE::Session and POE::NFA export conflicting constants"
-    if grep(/^(Session|NFA)$/, @sessions) > 1;
+    if scalar @sessions > 1;
 
   # If a session was specified, use that.  Otherwise use Session.
   if (@sessions) {
@@ -270,7 +270,7 @@ classes describe what's common among each class.  It's a good idea to at
 least skim the base class documentation since the subclasses tend not to
 rehash the common things.
 
-L<POE::Queue|POE::Queue>, L<POE::Resource|POE::Resource>, L<POE::Loop|POE::Loop>, and L<POE::API|POE::API> document the
+L<POE::Queue|POE::Queue>, L<POE::Resource|POE::Resource>, and L<POE::Loop|POE::Loop> document the
 concepts and sometimes the standard interfaces behind multiple
 subclasses.  You're encouraged to have a look.
 
@@ -365,10 +365,9 @@ see the GETTING HELP section if you run into a problem.
 POE is expected to work on most forms of UNIX, including FreeBSD,
 MacOS X, Linux, Solaris.  Maybe even AIX and QNX, but we're not sure.
 
-POE has also been tested on Windows XP, using the latest version of
-ActiveState Perl and Cygwin Perl.  Support for the Neopolitan Perls
-(Vanilla, Strawberry and Chocolate) is still early, but there should
-be no major show stoppers.
+POE is also tested on Windows XP, using the latest version of
+ActiveState, Strawberry and Cygwin Perl.  POE is fully supported with
+Strawberry Perl, as it's included in the Strawberry distribution.
 
 OS/2 and MacOS 9 have been reported to work in the past, but nobody
 seems to be testing there anymore.  Reports and patches are still
@@ -384,7 +383,7 @@ POE's quality is due in large part to the fine work of Chris Williams
 and the other CPAN testers.  They have dedicated resources towards
 ensuring CPAN distributions pass their own tests, and we watch their
 reports religiously.  You can, too.  The latest POE test reports can
-be found at <http://testers.cpan.org/search?request=dist&dist=POE>.
+be found at L<http://cpantesters.org/distro/P/POE.html>.
 
 Thanks also go out to Benjamin Smith and the 2006 Google Summer of
 Code.  Ben was awarded a grant to improve POE's test suite, which he
@@ -406,6 +405,26 @@ through contributions of time, patches, and other resources.  Some of
 them are: Sean Puckett, Douglas Couch, Andrew Chen, Uhlarik Ondoej,
 Nick Williams, and Chris Williams (no relation).
 
+=head2 Linux/Unix Issues
+
+=head3 pty woes
+
+Some distributions chose to not completely setup the pseudo-tty
+support. This is needed for L<POE::Wheel::Run> to interact with the
+subprocess. If you see something like this while running C<make test>
+please look at your distribution's documentation on how to fix it. For
+example, on Debian-based systems the solution was to execute 
+"sudo apt-get install udev".
+
+  t/30_loops/io_poll/wheel_run.t ..................... 1/99
+  pty_allocate(nonfatal): posix_openpt(): No such file or directory at /usr/local/lib/perl/5.10.0/IO/Pty.pm line 24.
+  ...
+  Cannot open a pty at /home/apoc/poe/blib/lib/POE/Wheel/Run.pm line 251
+  Compilation failed in require at t/30_loops/io_poll/wheel_run.t line 24.
+  # Looks like you planned 99 tests but ran 5.
+  # Looks like your test exited with 22 just after 5.
+  t/30_loops/io_poll/wheel_run.t ..................... Dubious, test returned 22 (wstat 5632, 0x1600)
+
 =head2 Other Compatibility Issues
 
 None currently known.  See GETTING HELP below if you've run into
@@ -495,8 +514,7 @@ L<POE::Queue>, L<POE::Queue::Array>
 
 L<POE::Resource>, L<POE::Resource::Aliases>, L<POE::Resource::Events>,
 L<POE::Resource::Extrefs>, L<POE::Resource::FileHandles>,
-L<POE::Resource::Performance>, L<POE::Resource::SIDs>,
-L<POE::Resource::Sessions>, L<POE::Resource::Signals>
+L<POE::Resource::SIDs>, L<POE::Resource::Sessions>, L<POE::Resource::Signals>
 
 =head2 Helpers
 
@@ -591,6 +609,9 @@ support different serialization methods.  He has also improved POE's quality
 by finding and fixing several bugs.  He provided POE a much needed code
 review around version 0.06.
 
+Lately, Philip tracked down the race condition in signal handling and
+fixed it with the signal pipe.
+
 =item Arnar M. Hrafnkelsson
 
 Arnar is <addi@umich.edu>.  Addi tested POE and L<POE::Component::IRC|POE::Component::IRC> on
@@ -628,7 +649,7 @@ problem relating to anonymous subs, scope and @{} processing.
 
 =item Matt Sergeant
 
-Matt contributed L<POE::Kernel::Poll|POE::Kernel::Poll>, a more efficient way to watch
+Matt contributed C<POE::Kernel::Poll>, a more efficient way to watch
 multiple files than select().  It's since been moved to
 L<POE::Loop::IO_Poll|POE::Loop::IO_Poll>.
 
