@@ -3,7 +3,7 @@ package POE::Component::Client::TCP;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.269_002'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.294_272'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Errno qw(ETIMEDOUT ECONNRESET);
@@ -378,7 +378,7 @@ POE::Component::Client::TCP implements a generic single-Session
 client.  Internally it uses POE::Wheel::SocketFactory to establish the
 connection and POE::Wheel::ReadWrite to interact with the server.
 
-POE::Component::Cilent::TCP is customized by providing callbacks for
+POE::Component::Client::TCP is customized by providing callbacks for
 common operations.  Most operations have sensible default callbacks,
 so clients may be created with as little work as possible.
 
@@ -405,7 +405,7 @@ C<Connected> callback to fire before transmitting data to the server.
 
 The client's constructor may seem to take a daunting number of
 parameters.  As with most POE modules, POE::Component::Client::TCP
-tries to do as much work in its constructor so that the runtime code
+tries to do as much work in its constructor so that the run-time code
 path is relatively light.
 
 =head3 Constructor Parameters Affecting the Session
@@ -608,8 +608,9 @@ C<Domain> sets the address or protocol family within which to operate.
 The C<Domain> may be any value that POE::Wheel::SocketFactory
 supports.  AF_INET (Internet address space) is used by default.
 
-Use AF_INET6 for IPv6 support.  This constant is exported by Socket6,
-which must be loaded B<before> POE::Component::Client::TCP.
+Use AF_INET6 for IPv6 support.  This constant is exported by Socket.
+Also be sure to have Socket::GetAddrInfo installed, which is required
+by POE::Wheel::SocketFactory for IPv6 support.
 
 =head4 RemoteAddress
 
@@ -701,7 +702,7 @@ is set.
 
 =head4 ServerInput
 
-C<ServerInput> is a requried callback.  It is called for each fully
+C<ServerInput> is a required callback.  It is called for each fully
 parsed input record received by POE::Wheel::ReadWrite.  $_[ARG0]
 contains the input record, the format of which is determined by the
 C<Filter> constructor parameter.
@@ -750,7 +751,7 @@ POE::Component::Client::TCP requires some heap space for its own
 bookkeeping.  The following members are used and should be used as
 directed, or with care.
 
-This sampe input handler is an example of most reserved heap members:
+This sample input handler is an example of most reserved heap members:
 
   sub handle_input {
     # Pending input from when we were connected.
@@ -767,9 +768,9 @@ This sampe input handler is an example of most reserved heap members:
 
 The read-only C<server> heap member contains the POE::Wheel object
 used to connect to or talk with the server.  While the component is
-connecting, C<server> will be a POE::Wheel::ReadWrite object.  After
-the connection has been made, it becomes a POE::Wheel::SocketFactory
-object.
+connecting, C<server> will be a L<POE::Wheel::SocketFactory> object.  After
+the connection has been made, C<server> is replaced with a
+L<POE::Wheel::ReadWrite> object.
 
 The most reliable way to avoid prematurely using C<server> is to first
 check the C<connected> reserved heap member.  See the example above.
