@@ -3,7 +3,7 @@ package POE::Component::Server::TCP;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.269_002'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.294_272'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
@@ -264,8 +264,9 @@ sub new {
                 $heap->{remote_ip} = inet_ntoa($remote_addr);
               }
               else {
-                $heap->{remote_ip} =
-                  Socket6::inet_ntop($domain, $remote_addr);
+                $heap->{remote_ip} = (
+                  Socket::GetAddrInfo::getaddrinfo($remote_addr)
+                )[1];
               }
 
               $heap->{remote_port} = $remote_port;
@@ -698,7 +699,7 @@ that manages such a hash.
   }
 
 The component's C<Started> callback is invoked at the end of the
-master session's startup routine.  The @_[ARG0..$#_] parameters are
+master session's start-up routine.  The @_[ARG0..$#_] parameters are
 set to a copy of the values in the server's C<ListenerArgs>
 constructor parameter.  The other parameters are standard for
 POE::Session's _start handlers.
@@ -706,8 +707,8 @@ POE::Session's _start handlers.
 The component's C<Error> callback is invoked when the server has a
 problem listening for connections.  C<Error> may also be called if the
 component's default acceptor has trouble accepting a connection.
-C<Error> receives the usual ones for POE::Wheel::SocketFactory and
-POE::Wheel::ReadWrite L<ErrorEvent>.
+C<Error> receives the usual ones for L<POE::Wheel::SocketFactory/FailureEvent> and
+L<POE::Wheel::ReadWrite/ErrorEvent>.
 
 =head2 Default Child Connection Sessions
 
@@ -813,7 +814,7 @@ C<Alias> parameter.
 
 POE::Component::Server::TCP does a lot of work in its constructor.
 The design goal is to push as much overhead into one-time construction
-so that ongoing runtime has less overhead.  Because of this, the
+so that ongoing run-time has less overhead.  Because of this, the
 server's constructor can take quite a daunting number of parameters.
 
 POE::Component::Server::TCP always returns a POE::Session ID for the
@@ -822,7 +823,7 @@ session that will be listening for new connections.
 Many of the constructor parameters have been previously described.
 They are covered briefly again below.
 
-=head3 Server Sesson Configuration
+=head3 Server Session Configuration
 
 These constructor parameters affect POE::Component::Server::TCP's main
 listening session.
@@ -838,7 +839,7 @@ C<SuccessEvent>.
 
 C<Acceptor> is optional; the default handler will create a new session
 for each connection.  All the "Client" constructor parameters are used
-to customize this session.  In other words, C<CleintInput> and such
+to customize this session.  In other words, C<ClientInput> and such
 B<are not used when C<Acceptor> is set>.
 
 The default C<Acceptor> adds significant convenience and flexibility
@@ -894,7 +895,7 @@ connections.  This may be useful if a server must perform lengthy
 initialization before allowing connections.  When the initialization
 finishes, it can yield(set_concurrency => -1) to enable connections.
 Likewise, a running server may yield(set_concurrency => 0) or any
-other number to dynamically tune its concurrency.  See L<EVENTS> for
+other number to dynamically tune its concurrency.  See L</EVENTS> for
 more about the set_concurrency event.
 
 Note: For C<Concurrency> to work with a custom C<Acceptor>, the
@@ -931,8 +932,9 @@ C<Domain> sets the address or protocol family within which to operate.
 The C<Domain> may be any value that POE::Wheel::SocketFactory
 supports.  AF_INET (Internet address space) is used by default.
 
-Use AF_INET6 for IPv6 support.  This constant is exported by Socket6,
-which must be loaded B<before> POE::Component::Server::TCP.
+Use AF_INET6 for IPv6 support.  This constant is exported by Socket.
+Also be sure to have Socket::GetAddrInfo installed, which is required
+by POE::Wheel::SocketFactory for IPv6 support.
 
 =head4 Error
 
@@ -956,8 +958,8 @@ transactions.
 C<Hostname> is the optional non-packed name of the interface the TCP
 server will bind to.  The hostname will always be resolved via
 inet_aton() and so can either be a dotted quad or a name.  Name
-resolution is a one-time startup action; there are no ongoing runtime
-penalties for using it.
+resolution is a one-time start-up action; there are no ongoing
+run-time penalties for using it.
 
 C<Hostname> guarantees name resolution, where C<Address> does not.
 It's therefore preferred to use C<Hostname> in cases where resolution
@@ -969,7 +971,7 @@ C<InlineStates> is optional.  If specified, it must hold a hashref of
 named callbacks.  Its syntax is that of POE:Session->create()'s
 inline_states parameter.
 
-Remember: These InlineStates handlers will be added to the client 
+Remember: These InlineStates handlers will be added to the client
 sessions, not to the main listening session.  A yield() in the listener
 will not reach these handlers.
 
@@ -1048,10 +1050,10 @@ client's session is started and ready for operation.  Banners are
 often sent to the remote client from this callback.
 
 The @_[ARG0..$#_] parameters to C<ClientConnected> are a copy of the
-values in the C<ClientArgs> constructor parameter's array referece.
+values in the C<ClientArgs> constructor parameter's array reference.
 The other @_ members are standard for a POE::Session _start handler.
 
-C<ClientConnected> is called once per session startup.  It will never
+C<ClientConnected> is called once per session start-up.  It will never
 be called twice for the same connection.
 
   ClientConnected => sub {
@@ -1155,7 +1157,7 @@ flushed before finishing a client shutdown.
 C<ClientInput> defines a per-connection callback to handle client
 input.  This callback receives its parameters directly from
 POE::Wheel::ReadWrite's C<InputEvent>.  ARG0 contains the input
-recored, the format of which is defined by C<ClientFilter> or
+record, the format of which is defined by C<ClientFilter> or
 C<ClientInputFilter>.  ARG1 has the wheel's unique ID, and so on.
 Please see POE:Wheel::ReadWrite for an in-depth description of
 C<InputEvent>.
@@ -1238,7 +1240,7 @@ in its L<Alias> parameter.
 =head3 disconnected
 
 The "disconnected" event informs the TCP server that a connection was
-closed.  It is needed when using L<Concurrency> with an L<Acceptor>
+closed.  It is needed when using L</Concurrency> with an L</Acceptor>
 callback.  The custom Acceptor must provide its own disconnect
 notification so that the server's connection counting logic works.
 
@@ -1248,7 +1250,7 @@ server will never know when clients have disconnected.
 =head3 set_concurrency
 
 "set_concurrency" set the number of simultaneous connections the
-server will be willing to accept.  See L<Concurrency> for more
+server will be willing to accept.  See L</Concurrency> for more
 details.  "set_concurrency" must have one parameter: the new maximum
 connection count.
 
@@ -1332,7 +1334,7 @@ procedure.
 
 =head3 remote_ip
 
-$_[HEAP]{remote_ip} contains the remote client's nuermic address in
+$_[HEAP]{remote_ip} contains the remote client's numeric address in
 human-readable form.
 
 =head3 remote_port
@@ -1343,7 +1345,7 @@ in human-readable form.
 =head3 remote_addr
 
 $_[HEAP]{remote_addr} contains the remote client's packed socket
-address in computer-readabe form.
+address in computer-readable form.
 
 =head3 shutdown
 
