@@ -3,7 +3,7 @@
 package POE::Resource::Events;
 
 use vars qw($VERSION);
-$VERSION = '1.269_002'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.294_272'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -48,6 +48,9 @@ sub _data_ev_finalize {
 
 ### Enqueue an event.
 
+sub FIFO_TIME_EPSILON () { 0.000001 }
+my $last_fifo_time = time();
+
 sub _data_ev_enqueue {
   my (
     $self,
@@ -66,8 +69,15 @@ sub _data_ev_enqueue {
   # This is awkward, but faster than using the fields individually.
   my $event_to_enqueue = [ @_[1..8] ];
 
+  my $new_id;
   my $old_head_priority = $kr_queue->get_next_priority();
-  my $new_id = $kr_queue->enqueue($time, $event_to_enqueue);
+
+  unless ($type & ET_MASK_DELAYED) {
+    $time = $last_fifo_time + FIFO_TIME_EPSILON if $time <= $last_fifo_time;
+    $last_fifo_time = $time;
+  }
+
+  $new_id = $kr_queue->enqueue($time, $event_to_enqueue);
 
   if (TRACE_EVENTS) {
     _warn(
@@ -78,7 +88,7 @@ sub _data_ev_enqueue {
     );
   }
 
-  if ($kr_queue->get_item_count() == 1) {
+  unless (defined $old_head_priority) {
     $self->loop_resume_time_watcher($time);
   }
   elsif ($time < $old_head_priority) {
@@ -88,11 +98,13 @@ sub _data_ev_enqueue {
   # This is the counterpart to _data_ev_refcount_dec().  It's only
   # used in one place, so it's not in its own function.
 
-  $event_count{$session}++;
-  $self->_data_ses_refcount_inc($session);
+  $self->_data_ses_refcount_inc($session) unless $event_count{$session}++;
 
-  $post_count{$source_session}++;
-  $self->_data_ses_refcount_inc($source_session);
+  return $new_id if $session == $source_session;
+
+  $self->_data_ses_refcount_inc($source_session) unless (
+    $post_count{$source_session}++
+  );
 
   return $new_id;
 }
@@ -102,24 +114,52 @@ sub _data_ev_enqueue {
 sub _data_ev_clear_session {
   my ($self, $session) = @_;
 
-  my $my_event = sub {
-    ($_[0]->[EV_SESSION] == $session) || ($_[0]->[EV_SOURCE] == $session)
-  };
+  # Events sent to the session.
+  PENDING: {
+    my $pending_count = $event_count{$session};
+    last PENDING unless $pending_count;
 
-  # TODO - This is probably incorrect.  The total event count will be
-  # artificially inflated for events from/to the same session.  That
-  # is, a yield() will count twice.
-  my $total_event_count = (
-    ($event_count{$session} || 0) +
-    ($post_count{$session} || 0)
-  );
+    foreach (
+      $kr_queue->remove_items(
+        sub { $_[0][EV_SESSION] == $session },
+        $pending_count
+      )
+    ) {
+      $self->_data_ev_refcount_dec(
+        @{$_->[ITEM_PAYLOAD]}[EV_SOURCE, EV_SESSION]
+      );
+      $pending_count--;
+    }
 
-  foreach ($kr_queue->remove_items($my_event, $total_event_count)) {
-    $self->_data_ev_refcount_dec(@{$_->[ITEM_PAYLOAD]}[EV_SOURCE, EV_SESSION]);
+    last PENDING unless $pending_count;
+
+    croak "lingering pending count: $pending_count" if $pending_count;
   }
 
-  croak if delete $event_count{$session};
-  croak if delete $post_count{$session};
+  # Events sent by the session.
+  SENT: {
+    my $sent_count = $post_count{$session};
+    last SENT unless $sent_count;
+
+    foreach (
+      $kr_queue->remove_items(
+        sub { $_[0][EV_SOURCE] == $session },
+        $sent_count
+      )
+    ) {
+      $self->_data_ev_refcount_dec(
+        @{$_->[ITEM_PAYLOAD]}[EV_SOURCE, EV_SESSION]
+      );
+      $sent_count--;
+    }
+
+    last SENT unless $sent_count;
+
+    croak "lingering sent count: $sent_count" if $sent_count;
+  }
+
+  croak "lingering event count" if delete $event_count{$session};
+  croak "lingering post count" if delete $post_count{$session};
 }
 
 # TODO Alarm maintenance functions may move out to a separate
@@ -200,14 +240,21 @@ sub _data_ev_refcount_dec {
 
   if (ASSERT_DATA) {
     _trap $dest_session unless exists $event_count{$dest_session};
+  }
+
+  $self->_data_ses_refcount_dec($dest_session) unless (
+    --$event_count{$dest_session}
+  );
+
+  return if $dest_session == $source_session;
+
+  if (ASSERT_DATA) {
     _trap $source_session unless exists $post_count{$source_session};
   }
 
-  $event_count{$dest_session}--;
-  $self->_data_ses_refcount_dec($dest_session);
-
-  $post_count{$source_session}--;
-  $self->_data_ses_refcount_dec($source_session);
+  $self->_data_ses_refcount_dec($source_session) unless (
+    --$post_count{$source_session}
+  );
 }
 
 ### Fetch the number of pending events sent to a session.
@@ -241,9 +288,10 @@ sub _data_ev_dispatch_due {
 
   my $now = time();
   my $next_time;
-  while (defined($next_time = $kr_queue->get_next_priority())) {
-    last if $next_time > $now;
-
+  while (
+    defined($next_time = $kr_queue->get_next_priority()) and
+    $next_time <= $now
+  ) {
     my ($due_time, $id, $event) = $kr_queue->dequeue_next();
 
     if (TRACE_EVENTS) {
@@ -255,19 +303,22 @@ sub _data_ev_dispatch_due {
     # being dispatched.  As far as I can tell, all events will be
     # "blocked" according to these rules.
 
-    if ($due_time < $now) {
-      $self->_data_stat_add('blocked', 1);
-      $self->_data_stat_add('blocked_seconds', $now - $due_time);
+    if (TRACE_STATISTICS) {
+      if ($due_time < $now) {
+        $self->_data_stat_add('blocked', 1);
+        $self->_data_stat_add('blocked_seconds', $now - $due_time);
+      }
     }
 
-    # TODO - Can these two lines be reversed?
-    # TODO - May avoid entering/removing GC mark entries.
+    # TODO - Why can't we reverse these two lines?
+    # TODO - Reversing them could avoid entering and removing GC marks.
     $self->_data_ev_refcount_dec($event->[EV_SOURCE], $event->[EV_SESSION]);
     $self->_dispatch_event(@$event, $due_time, $id);
 
     # Stop the system if an unhandled exception occurred.
     # This wipes out all sessions and associated resources.
-    POE::Kernel->stop() if $POE::Kernel::kr_exception;
+    next unless $POE::Kernel::kr_exception;
+    POE::Kernel->stop();
   }
 
   # Sweep for dead sessions.  The sweep may alter the next queue time.
