@@ -3,7 +3,7 @@ package POE::Session;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.269_002'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.294_272'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Errno;
@@ -813,7 +813,7 @@ dispatch interface to those handlers.
 Consider the L</SYNOPSIS> for example.  A POE::Session instance is
 created with two C<inline_states>, each mapping an event name
 ("_start" and "next") to an inline subroutine.  POE::Session ensures
-that C<$_[KERNEL]> and so on are meaningful within an event handler.
+that L</$_[KERNEL]> and so on are meaningful within an event handler.
 
 Event handlers may also be object or class methods, using
 L</object_states> and L</package_states> respectively.  The create()
@@ -824,12 +824,12 @@ Notice that the created POE::Session object has not been saved to a
 variable.  The new POE::Session object gives itself to POE::Kernel,
 which then manages it and all the resources it uses.
 
-It's possible to keep references to new POE::Session objects, but it's 
-not usually necessary.  If an application is not careful about 
-cleaning up these references you will create circular references, 
-which will leak memory when POE::Kernel would normally destroy 
-the POE::Session object.  It is recommended that you keep 
-the session's C</ID> instead.
+It's possible to keep references to new POE::Session objects, but it's
+not usually necessary.  If an application is not careful about
+cleaning up these references you will create circular references,
+which will leak memory when POE::Kernel would normally destroy the
+POE::Session object.  It is recommended that you keep the session's
+L<ID> instead.
 
 =head2 POE::Session's Calling Convention
 
@@ -841,16 +841,17 @@ POE::Session's unconventional calling convention.  For example:
     ...;
   }
 
-Or the use of CL</$_[KERNEL]>, CL</$_[HEAP]> and CL</$_[ARG0]> inline, as is done
-in most examples.
+Or the use of C<$_[KERNEL]>, C<$_[HEAP]> and C<$_[ARG0]> inline,
+as is done in most examples.
 
 What's going on here is rather basic.  Perl passes parameters into
-subroutines or methods using the @_ array.  C<KERNEL>, C<HEAP>, C<ARG0> and
-others are constants exported by POE::Session (which is included for
-free when a program uses POE).
+subroutines or methods using the @_ array.  C<KERNEL>, C<HEAP>,
+C<ARG0> and others are constants exported by POE::Session (which is
+included for free when a program uses POE).
 
-So CL</$_[KERNEL]> is an event handler's KERNELth parameter.  C<@_[HEAP,
-ARG0]> is a slice of @_ containing the HEAPth and ARG0th parameters.
+So C<$_[KERNEL]> is an event handler's KERNELth parameter.
+C<@_[HEAP, ARG0]> is a slice of @_ containing the HEAPth and ARG0th
+parameters.
 
 While this looks odd, it's perfectly plain and legal Perl syntax.  POE
 uses it for a few reasons:
@@ -864,8 +865,8 @@ hash or array references and then dereferencing them in the handler.
 
 =item 2
 
-Typos in hash-based parameter lists are either subtle runtime
-errors or requires constant runtime checking.  Constants are either
+Typos in hash-based parameter lists are either subtle run-time
+errors or requires constant run-time checking.  Constants are either
 known at compile time, or are clear compile-time errors.
 
 =item 3
@@ -882,7 +883,7 @@ use only the parameters they're interested in.
 
 =head2 POE::Session Parameters
 
-Event handlers receive most of their runtime context in up to nine
+Event handlers receive most of their run-time context in up to nine
 callback parameters.  POE::Kernel provides many of them.
 
 =head3 $_[OBJECT]
@@ -905,11 +906,14 @@ methods in the same object.  C<$_[OBJECT]> helps this happen.
     $self->update_status_line();
   }
 
-You may also use method inheritance
+You may also use method inheritance.  Here we invoke
+$self->a_method(@_).  Since Perl's C<<->>> operator unshifts $self
+onto the beginning of @_, we must first shift a copy off to maintain
+POE's parameter offsets:
 
-  sub Amethod {
+  sub a_method {
     my $self = shift;
-    $self->SUPER::Amethod( @_ );
+    $self->SUPER::a_method( @_ );
     # ... more work ...
   }
 
@@ -917,11 +921,11 @@ You may also use method inheritance
 
 C<$_[SESSION]> is a reference to the current session object.  This lets event
 handlers access their session's methods.  Programs may also compare
-C<$_[SESSION]> to CL</$_[SENDER]> to verify that intra-session events did not
+C<$_[SESSION]> to C<$_[SENDER]> to verify that intra-session events did not
 come from other sessions.
 
 C<$_[SESSION]> may also be used as the destination for intra-session
-CL<post()|POE::Kernel/post> and CL<call()|POE::Kernel/call>.  CL<yield()|POE::Kernel/yield> is marginally more convenient and
+L<post()|POE::Kernel/post> and L<call()|POE::Kernel/call>.  L<yield()|POE::Kernel/yield> is marginally more convenient and
 efficient than C<post($_[SESSION], ...)> however.
 
 It is bad form to access another session directly.  The recommended
@@ -1078,7 +1082,7 @@ means anything stored there will be available to any other event
 handler regardless of the object.
 
 event_2a is handled by calling C<< $object_2->method_2a(...) >>.  In this
-case CL</$_[OBJECT]> is $object_2.  C<$_[HEAP]> is the same anonymous hashref
+case C<$_[OBJECT]> is $object_2.  C<$_[HEAP]> is the same anonymous hashref
 that was passed to the event_1a handler, though.  The methods are resolved
 when the event is handled (late-binding).
 
@@ -1118,7 +1122,7 @@ object upon success, but most applications won't need to save it.
 C<create()> invokes the newly started session's _start event handler
 before returning.
 
-C<create()> also passes the new POE::Session object to L</POE::Kernel>.
+C<create()> also passes the new POE::Session object to L<POE::Kernel>.
 POE's kernel holds onto the object in order to dispatch events to it.
 POE::Kernel will release the object when it detects the object has
 become moribund.  This should cause Perl to destroy the object if
@@ -1239,7 +1243,7 @@ triggers $object->_stop():
   );
 
 Methods are verified when the session is created, but also resolved when the
-handler is called (late binding).  Most of the time, a method won't change. 
+handler is called (late binding).  Most of the time, a method won't change.
 But in some circumstance, such as dynamic inheritance, a method could
 resolve to a different subroutine.
 
@@ -1247,7 +1251,7 @@ resolve to a different subroutine.
 
 POE::Session sessions support a small number of options, which may be
 initially set with the C<option> constructor parameter and changed at
-runtime with the C<option()|/option> mehtod.
+run time with the C<option()|/option> mehtod.
 
 C<option> takes a hashref with option =E<gt> value pairs:
 
@@ -1496,7 +1500,7 @@ Rather, helper functions may use:
   use POE::Kernel; # exports $poe_kernel
   sub some_helper_function {
     my (@specific_parameters) = @_;
-    my $session = $kernel->get_active_session();
+    my $session = $poe_kernel->get_active_session();
     my $heap = $session->get_heap();
   }
 
@@ -1519,7 +1523,7 @@ parameters to C<create()>.
 
 Any parameters not recognized by C<create()> must be removed from the
 CREATE_PARAMETERS before C<instantiate()> returns.  C<create()> will
-L<croak|Carp/croak>
+L<croak|Carp>
 if it discovers unknown parameters.
 
 Be sure to return C<$self> from instantiate.
@@ -1715,7 +1719,7 @@ to post the event back to the current session:
     $_[KERNEL]->yield( $next );
   }
 
-Or the L<$_[SESSION]> parameter passed to every event handler, but
+Or the L</$_[SESSION]> parameter passed to every event handler, but
 yield() is more efficient.
 
   next => sub {
@@ -1724,8 +1728,8 @@ yield() is more efficient.
     $_[KERNEL]->post( $_[SESSION], $next );
   }
 
-Along the same lines as L<$_[SESSION]>, a session can respond back to
-the sender of an event by posting to L<$_[SENDER]>.  This is great for
+Along the same lines as L</$_[SESSION]>, a session can respond back to
+the sender of an event by posting to L</$_[SENDER]>.  This is great for
 responding to requests.
 
 If a program must hold onto some kind of dynamic session reference,
