@@ -5,12 +5,12 @@ package POE::Wheel::Curses;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.269_002'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.294_272'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(croak);
 use Curses qw(
   initscr start_color cbreak raw noecho nonl nodelay timeout keypad
-  intrflush meta typeahead mousemask ALL_MOUSE_EVENTS clear refresh
+  intrflush meta typeahead clear refresh
   endwin COLS
 );
 use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
@@ -65,9 +65,6 @@ sub new {
   meta(1);
   typeahead(-1);
 
-  my $old_mouse_events = 0;
-  mousemask(ALL_MOUSE_EVENTS, $old_mouse_events);
-
   clear();
   refresh();
 
@@ -226,6 +223,9 @@ Curses::keyname() on the keystroke.
 
 C<$_[ARG1]> contains the POE::Wheel::Curses object's ID.
 
+Mouse events aren't portable.  As of October 2009, it's up to the
+application to decide whether to call mousemask().
+
 =head1 SEE ALSO
 
 L<Curses> documents what can be done with Curses.  Also see the man
