@@ -3,19 +3,24 @@ package POE::Wheel::SocketFactory;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.269_002'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.294_272'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw( carp croak );
 use Symbol qw( gensym );
 
 use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
-use Errno qw(EWOULDBLOCK EADDRNOTAVAIL EINPROGRESS EADDRINUSE);
+use Errno qw(
+  EWOULDBLOCK EADDRNOTAVAIL EINPROGRESS EADDRINUSE ECONNABORTED
+  ESPIPE
+);
+
 use Socket qw(
-  AF_INET SOCK_STREAM SOL_SOCKET AF_UNIX PF_UNIX 
-  PF_INET SOCK_DGRAM SO_ERROR unpack_sockaddr_in 
+  AF_INET AF_INET6 SOCK_STREAM SOL_SOCKET AF_UNIX PF_UNIX 
+  PF_INET PF_INET6 SOCK_DGRAM SO_ERROR unpack_sockaddr_in 
   unpack_sockaddr_un PF_UNSPEC SO_REUSEADDR INADDR_ANY 
   pack_sockaddr_in pack_sockaddr_un inet_aton SOMAXCONN
 );
+
 use IO::Handle ();
 use FileHandle ();
 use POE qw( Wheel );
@@ -47,16 +52,12 @@ sub MY_SOCKET_SELECTED () { 12 }
 # Test and provide for each constant separately, per suggestion in
 # rt.cpan.org 27250.
 BEGIN {
-  eval { require Socket6 };
+  eval "use Socket::GetAddrInfo qw(:newapi getaddrinfo getnameinfo)";
   if ($@) {
-    *Socket6::AF_INET6 = sub () { ~0 };
-    *Socket6::PF_INET6 = sub () { ~0 };
-  }
-  else {
-    eval { my $x = &Socket6::AF_INET6 };
-    *Socket6::AF_INET6 = sub () { ~0 } if $@;
-    eval { my $x = &Socket6::PF_INET6 };
-    *Socket6::PF_INET6 = sub () { ~0 } if $@;
+    my $why = $@;
+    $why =~ s/ at \(eval.* line 1\.\s*//;
+    *getaddrinfo = sub { return "Socket::GetAddrInfo not loaded: $why" };
+    *getnameinfo = sub { return "Socket::GetAddrInfo not loaded: $why" };
   }
 }
 
@@ -73,8 +74,8 @@ sub DOM_INET6 () { 'inet6' }  # INET v6 domain socket
 my %map_family_to_domain = (
   AF_UNIX,  DOM_UNIX,  PF_UNIX,  DOM_UNIX,
   AF_INET,  DOM_INET,  PF_INET,  DOM_INET,
-  &Socket6::AF_INET6, DOM_INET6,
-  &Socket6::PF_INET6, DOM_INET6,
+  AF_INET6, DOM_INET6,
+  PF_INET6, DOM_INET6,
 );
 
 sub SVROP_LISTENS () { 'listens' }  # connect/listen sockets
@@ -171,7 +172,8 @@ sub _define_accept_state {
         }
         elsif ( $domain eq DOM_INET6 ) {
           $peer = getpeername($new_socket);
-          ($peer_port, $peer_addr) = Socket6::unpack_sockaddr_in6($peer);
+          ((my $error), $peer_port, $peer_addr) =  getnameinfo($peer);
+          warn $error if $error;
         }
         else {
           die "sanity failure: socket domain == $domain";
@@ -182,7 +184,8 @@ sub _define_accept_state {
           $unique_id
         );
       }
-      elsif ($! != EWOULDBLOCK) {
+      elsif ($! != EWOULDBLOCK and $! != ECONNABORTED and $! != ESPIPE) {
+        # OSX reports ESPIPE, which isn't documented anywhere.
         $$event_failure && $k->call(
           $me, $$event_failure,
           'accept', ($!+0), $!, $unique_id
@@ -289,10 +292,9 @@ sub _define_connect_state {
       # INET6 socket stacks tend not to.
       elsif ($domain eq DOM_INET6) {
         if (defined $peer) {
-          eval {
-            ($peer_port, $peer_addr) = Socket6::unpack_sockaddr_in6($peer);
-          };
-          if (length $@) {
+          ((my $error), $peer_port, $peer_addr) = getnameinfo($peer);
+          if ($error) {
+            warn $error;
             $peer_port = $peer_addr = undef;
           }
         }
@@ -776,7 +778,7 @@ sub new {
       $bind_address = (
         (defined $params{BindAddress})
         ? $params{BindAddress}
-        : Socket6::in6addr_any()
+        : "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"  # XXX - Only Socket6 has?
       );
 
       # Set the bind port, or default to 0 (any) if none specified.
@@ -798,12 +800,16 @@ sub new {
       BEGIN { eval { require bytes } and bytes->import; }
 
       # Resolve the bind address.
-      my @info = Socket6::getaddrinfo(
-        $bind_address, $bind_port,
-        $self->[MY_SOCKET_DOMAIN], $self->[MY_SOCKET_TYPE],
+      my ($error, @addresses) = getaddrinfo(
+        $bind_address, $bind_port, {
+          family   => $self->[MY_SOCKET_DOMAIN],
+          socktype => $self->[MY_SOCKET_TYPE],
+        }
       );
 
-      if (@info < 5) {  # unless defined $bind_address
+      unless (@addresses) {
+        warn $error if $error;
+
         $! = EADDRNOTAVAIL;
         $poe_kernel->yield(
           $event_failure,
@@ -812,7 +818,7 @@ sub new {
         return $self;
       }
 
-      $bind_address = $info[3];
+      $bind_address = $addresses[0]->{addr};
     }
   }
 
@@ -900,16 +906,19 @@ sub new {
         $error_tag = "inet_aton";
       }
       elsif ($abstract_domain eq DOM_INET6) {
-        my @info = Socket6::getaddrinfo(
-          $params{RemoteAddress}, $remote_port,
-          $self->[MY_SOCKET_DOMAIN], $self->[MY_SOCKET_TYPE],
+        my ($error, @addresses) = getaddrinfo(
+          $params{RemoteAddress}, $remote_port, {
+            family   => $self->[MY_SOCKET_DOMAIN],
+            socktype => $self->[MY_SOCKET_TYPE],
+          },
         );
 
-        if (@info < 5) {
+        unless (@addresses) {
+          warn $error if $error;
           $connect_address = undef;
         }
         else {
-          $connect_address = $info[3];
+          $connect_address = $addresses[0]->{addr};
         }
 
         $error_tag = "getaddrinfo";
@@ -1222,7 +1231,7 @@ work where establishing connections normally would block.
 
 =head2 new
 
-new() creats a new POE::Wheel::SocketFactory object.  For sockets
+new() creates a new POE::Wheel::SocketFactory object.  For sockets
 which listen() for and accept() connections, the wheel will generate
 new sockets for each accepted client.  Socket factories for one-shot
 sockets, such as UDP peers or clients established by connect() only
@@ -1253,9 +1262,12 @@ POE::Wheel::SocketFactory contains a table of supported domains and
 the instructions needed to create them.  Please send patches to
 support additional domains, as needed.
 
-Note: C<AF_INET6> and C<PF_INET6> are supplied by the L<Socket6>
-module, which is available on the CPAN.  You must have Socket6 loaded
-before SocketFactory can create IPv6 sockets.
+Note: C<AF_INET6> and C<PF_INET6> are supplied by the L<Socket>
+module included in Perl 5.8.0 or later.  Perl versions before 5.8.0
+should not attempt to use IPv6 until someone contributes a workaround.
+
+IPv6 support requires a 21st century Socket module and the presence of
+Socket::GetAddrInfo to resolve host names to IPv6 addresses.
 
 TODO - Example.
 
@@ -1350,7 +1362,7 @@ TODO - Example.
 
 =head3 Connecting the Socket to a Remote Endpoint
 
-A socket may either listen for connections to arrive, intitate
+A socket may either listen for connections to arrive, initiate
 connections to a remote endpoint, or be connectionless (such as in the
 case of UDP sockets).
 
@@ -1435,14 +1447,14 @@ client.
 =head4 FailureEvent
 
 C<FailureEvent> names the event POE::Wheel::SocketFactory will emit
-whenever something goes wrong.  It ususally represents some kind of
+whenever something goes wrong.  It usually represents some kind of
 built-in function call error.  See L</PUBLIC EVENTS> for details, as
 some errors are handled internally by this wheel.
 
 =head2 event
 
 event() allows a session to change the events emitted by a wheel
-without destrying and re-creating the wheel.  It accepts one or more
+without destroying and re-creating the wheel.  It accepts one or more
 of the events listed in L</PUBLIC EVENTS>.  Undefined event names
 disable those events.
 
@@ -1518,7 +1530,9 @@ listening or connecting.  See below for the differences.
 
 For INET sockets, C<$_[ARG1]> and C<$_[ARG2]> hold the socket's remote
 address and port, respectively.  The address is packed; see
-L<Socket/inet_nota()> if a human-readable version is needed.
+L<Socket/inet_ntoa> if a human-readable IPv4 address is needed.
+L<Socket::GetAddrInfo/getnameinfo> provides numeric addresses for IPv4
+and IPv6 addresses.
 
   sub handle_new_client {
     my $accepted_socket = $_[ARG0];
@@ -1557,7 +1571,8 @@ holds a Perl built-in function name.
 
 C<$_[ARG1]> and C<$_[ARG2]> describe how the operation failed.  They
 contain the numeric and stringified versions of C<$!>, respectively.
-An application cannot merely check C<$!> because
+An application cannot merely check the global C<$!> variable since it
+may change during event dispatch.
 
 Finally, C<$_[ARG3]> contains the ID for the POE::Wheel::SocketFactory
 instance that generated the event.  See L</ID> and L<POE::Wheel/ID>
@@ -1576,9 +1591,10 @@ A sample FailureEvent handler:
 L<POE::Wheel> describes the basic operations of all wheels in more
 depth.  You need to know this.
 
-L<Socket6> is required for IPv6 work.  POE::Wheel::SocketFactory will
-load it automatically if it's installed, but applications will need to
-use it themselves to get access to AF_INET6.
+L<Socket::GetAddrInfo> is required for IPv6 work.
+POE::Wheel::SocketFactory will load it automatically if it's
+installed.  SocketDomain => AF_INET6 is required to trigger IPv6
+behaviors.  AF_INET6 is exported by the Socket module.
 
 The SEE ALSO section in L<POE> contains a table of contents covering
 the entire POE distribution.
