@@ -25,9 +25,7 @@ use vars qw(@EXPORT_OK);
 
 
 sub CONFIG_REQUIREMENTS () {
-  (
-    "POE::Test::Loops"  => '1.022_001',
-  )
+  ("POE::Test::Loops"  => '1.036')
 }
 
 sub CORE_REQUIREMENTS () {
@@ -46,6 +44,9 @@ sub CORE_REQUIREMENTS () {
       ? (
         "Win32::Console" => 0.031,
         "Win32API::File" => 0.05,
+        "Win32::Job"     => 0.03,
+        "Win32::Process" => 0,
+        "Win32"          => 0,
       )
       : (
         "IO::Tty"        => 1.08, # avoids crashes on fbsd
@@ -64,23 +65,31 @@ sub DIST_AUTHOR () {
 }
 
 sub DIST_ABSTRACT () {
-  ( 'A portable networking and multitasking framework.' )
+  ( 'Portable, event-loop agnostic eventy networking and multitasking.' )
 }
 
 sub CLEAN_FILES () {
   my @clean_files = qw(
+    */*/*/*/*~
+    */*/*/*~
+    */*/*/*~
+    */*/*~
+    */*~
+    *~
+    META.yml
+    Makefile.old
     bingos-followtail
     coverage.report
     poe_report.xml
     run_network_tests
-    test-output.err
     t/20_resources/10_perl
     t/20_resources/10_perl/*
     t/20_resources/20_xs
     t/20_resources/20_xs/*
-    t/30_loops/*/*
-    t/30_loops/*
     t/30_loops
+    t/30_loops/*
+    t/30_loops/*/*
+    test-output.err
   );
   "@clean_files";
 }
@@ -95,11 +104,11 @@ sub TEST_FILES () {
 }
 
 sub REPOSITORY () {
-  ( 'https://poe.svn.sourceforge.net/svnroot/poe/trunk' )
+  'https://github.com/rcaputo/poe'
 }
 
 sub HOMEPAGE () {
-  ( 'http://poe.perl.org/' )
+  'http://poe.perl.org/'
 }
 
 1;
